@@ -8,15 +8,17 @@ use parent qw/Class::Accessor Exporter/;
 __PACKAGE__->follow_best_practice;
 __PACKAGE__->mk_accessors(qw/aws_secret_access_key aws_access_key_id token region
 			     user_agent profile debug expiration role container order 
-			     serialized logger
+			     serialized logger timeout
 			    /);
 
 use Carp;
 use Data::Dumper;
 use Date::Format;
 use Exporter;
+use File::chdir;
+use File::HomeDir;
 use HTTP::Request;
-use JSON;
+use JSON::PP;
 use LWP::UserAgent;
 use POSIX::strptime qw/strptime/;
 use Time::Local;
@@ -28,7 +30,7 @@ use constant  AWS_CONTAINER_CREDENTIALS_URL          => 'http://169.254.170.2';
 
 use vars qw/$VERSION @EXPORT/;
 
-$VERSION = '1.0.11'; $VERSION=~s/\-.*$//;
+$VERSION = '1.0.12'; $VERSION=~s/\-.*$//;
 
 @EXPORT = qw/$VERSION/;
 
@@ -37,9 +39,12 @@ $VERSION = '1.0.11'; $VERSION=~s/\-.*$//;
   no strict 'refs';
   
   *{'Amazon::Credentials::Logger::debug'} = sub {
-    shift;
+    my $self = shift;
+
+    return if ! $self->{debug};
+    
     my @tm = localtime(time);
-    print STDERR sprintf("%s [%s] %s", strftime("%c", @tm), $$, @_);
+    print STDERR sprintf(" %s [%s] %s", strftime("%c", @tm), $$, @_);
   };
 }
 
@@ -193,6 +198,11 @@ Default region. The class will attempt to find the region in either
 the configuration files or the instance unless you specify the region
 in the constructor.
 
+=item timeout
+
+When looking for credentials in metadata URLs, this parameter
+specifies the timeout value for C<LWP>.  The default is 3 seconds.
+
 =back
 
 =cut
@@ -200,25 +210,28 @@ in the constructor.
 sub new {
   my $class = shift;
   my $self = $class->SUPER::new(ref($_[0]) ? $_[0] : { @_ });
-  
-  unless ( $self->get_logger ) {
-    $self->set_logger(bless {}, 'Amazon::Credentials::Logger');
+
+  if ( !$self->get_logger ) {
+    $self->set_logger(bless { debug => $self->get_debug }, 'Amazon::Credentials::Logger');
   }
 
-  unless ($self->get_user_agent) {
-    $self->set_user_agent(LWP::UserAgent->new());
+  $self->get_logger->debug("debug is on\n");
+  
+  if ( !$self->get_user_agent) {
+    # set a very low timeout
+    $self->set_user_agent(LWP::UserAgent->new(timeout => $self->get_timeout || 3));
   }
 
   $self->set_profile($ENV{AWS_PROFILE})
-    unless $self->get_profile;
+    if  !$self->get_profile;
 
-  $self->set_region($ENV{AWS_REGION} || $self->get_default_region)
-    unless $self->get_region;
-
-  unless ( $self->get_aws_secret_access_key && $self->get_aws_access_key_id ) {
+  if ( !$self->get_aws_secret_access_key || !$self->get_aws_access_key_id ) {
     $self->set_credentials;
   }
 
+  $self->set_region($ENV{AWS_REGION} || $ENV{AWS_DEFAULT_REGION} || $self->get_default_region)
+    if !$self->get_region;
+  
   $self;
 }
 
@@ -301,120 +314,129 @@ sub get_ec2_credentials {
 }
 
 sub find_credentials {
-  my $self = shift;
-  my %options = ref($_[0]) ? %{$_[0]} : @_;
+  my $self    = shift;
+  my %options = ref( $_[0] ) ? %{ $_[0] } : @_;
+
+  $self->get_logger->debug("find credentials\n");
   
-  $options{order} = $self->get_order || [ qw/env role container file/ ];
+  $options{order}   = $self->get_order  || [qw/env role container file/];
   $options{profile} = $options{profile} || $self->get_profile;
 
-  if (defined $options{profile} && $options{profile}) {
+  if ( defined $options{profile} && $options{profile} ) {
     $options{order} = ['file'];
   }
-  
-  my $creds = {};
-    
-  foreach (@{$options{order}}) {
-    /env/ && do {
-      if ( $ENV{AWS_ACCESS_KEY_ID} && $ENV{AWS_SECRET_ACCESS_KEY} ) {
-	@{$creds}{qw/source aws_access_key_id aws_secret_access_key token/} =
-	  ('ENV',@ENV{qw/AWS_ACCESS_KEY_ID AWS_SECRET_ACCESS_KEY AWS_SESSION_TOKEN/});
-	last;
-      }
-    };
 
-    /container/ && do {
+  my $creds = {};
+  my @order = @{ $options{order} };
+
+  foreach my $location (@order) {
+    $self->get_logger->debug("looking for credentials in $location\n");
+    
+    if ( $location eq 'env' ) {
+      if ( $ENV{AWS_ACCESS_KEY_ID} && $ENV{AWS_SECRET_ACCESS_KEY} ) {
+        @{$creds}{qw/source aws_access_key_id aws_secret_access_key token/}
+          = (
+          'ENV',
+          @ENV{qw/AWS_ACCESS_KEY_ID AWS_SECRET_ACCESS_KEY AWS_SESSION_TOKEN/}
+          );
+      } ## end if ( $ENV{AWS_ACCESS_KEY_ID...})
+    } ## end if ( $location eq 'env')
+    elsif ( $location eq 'container' ) {
       $creds = $self->get_creds_from_container();
-      last if ! $@ && $creds->{container};
-    };
-    
-    /role/ && do {
+      $self->get_logger->debug(Dumper [$creds]);
+    }
+    elsif ( $location eq 'role' ) {
       $creds = $self->get_creds_from_role();
-      last if ! $@ && $creds->{role};
-    };
-    
-    /file/ && do {
+    }
+    elsif ( $location eq 'file' ) {
       # look for ~/.aws/config and/or .aws/credentials
-      use File::chdir;
-      use File::HomeDir;
+
+      # in production environments, we should never be reading creds
+      # from files, so let's require these (not use)
 
       foreach my $config ( ".aws/config", ".aws/credentials" ) {
-	# reset this since we have hav
-	my $profile_name = $options{profile};
-	
-	local $CWD = home;
-	next unless -e $config;
-	
-	open my $fh, '<', $config
+        # reset this since we have hav
+        my $profile_name = $options{profile};
+
+        local $CWD = home;
+        
+        $self->get_logger->debug("home: [" . $CWD . "]\n");
+        
+        next unless -e $config;
+
+        open my $fh, '<', $config
           or croak 'could not open credentials file!';
-	
-	my $last_profile = '';
-	my $profile_to_find = $profile_name;
-	
-	# look for credentials...by interating through credentials file
-	while (<$fh>) {
-	  chomp;
-          my $current_line = $_;
-	  # once we find a profile section that matches, undef it
-	  # ./aws/credentials uses [profile-name]
-	  # ./aws/config uses [profile profile-name]
-	  
-	  if (/^\s*region\s*=\s*(.*?)\s*$/ ) {
+
+        my $last_profile    = '';
+        my $profile_to_find = $profile_name;
+
+        # look for credentials...by interating through credentials file
+        while (my $current_line = <$fh>) {
+          chomp $current_line;
+
+          $self->get_logger->debug("line: $current_line\n");
+          
+          # once we find a profile section that matches, undef it
+          # ./aws/credentials uses [profile-name]
+          # ./aws/config uses [profile profile-name]
+
+          if ($current_line =~/^\s*region\s*=\s*(.*?)\s*$/) {
             my $region = $1;
-	    # go ahead and use this region setting IF:
-	    # 1. this is the default profile (we may reset region later though)
-	    # 2. the profile we want to use is this profile
-	    # 3. we are not in a profile at all (this is yet another default)
-	    if ( $last_profile =~/default/ ||
-		 ($profile_to_find && $last_profile =~/$profile_to_find/) ||
-		 ! $last_profile ) {
-	      $self->set_region($region);
-	    }
-	  }
-	  
-	  if ( $profile_name ) {
-	    if (/^\s*\[\s*profile\s+$profile_name\s*\]/) {
-	      undef $profile_name;
-	    }
-	    elsif (/^\s*\[\s*$profile_name\s*\]/) {
-	      undef $profile_name;
-	      $last_profile = $current_line;
-	    }
-	  } 
-	  elsif (defined $profile_name && /^\s*\[\s*profile\s+/) {
-	    last;
-	  }
-	  elsif (/^\s*aws_secret_access_key\s*=\s*(.*)$/) {
-	    last if defined $creds->{aws_secret_access_key}; # next profile
-	    $creds->{aws_secret_access_key} = $1;
-	  }
-	  elsif (/^\s*aws_access_key_id\s*=\s*(.*)$/) {
-	    last if defined $creds->{aws_access_key_id}; # next profile
-	    $creds->{aws_access_key_id} = $1;
-	  }
-	  elsif (/^\s*aws_session_token\s*=\s*(.*)$/) {
-	    last if defined $creds->{token};
-	    $creds->{token} = $1;
-	  }
-	}
-	  
-	close $fh;
-	  
-	$self->get_logger->debug(Dumper [ $creds ])
-	  if $self->get_debug;
-	
-	$creds->{source} = $config if $creds->{aws_secret_access_key} && $creds->{aws_access_key_id};
-      }
-	
-      last if $creds->{source};
-    };
+            # go ahead and use this region setting IF:
+            # 1. this is the default profile (we may reset region later though)
+            # 2. the profile we want to use is this profile
+            # 3. we are not in a profile at all (this is yet another default)
+            if ( $last_profile =~ /default/
+              || ( $profile_to_find && $last_profile =~ /$profile_to_find/ )
+              || !$last_profile ) {
+              $self->set_region($region);
+            }
+          }
+
+          if ($profile_name) {
+            if ($current_line =~ /^\s*\[\s*profile\s+$profile_name\s*\]/) {
+              undef $profile_name;
+            }
+            elsif ($current_line =~/^\s*\[\s*$profile_name\s*\]/) {
+              undef $profile_name;
+              $last_profile = $current_line;
+            }
+          } ## end if ($profile_name)
+          elsif ( defined $profile_name && $current_line =~/^\s*\[\s*profile\s+/ ) {
+            last;
+          }
+          elsif ($current_line =~/^\s*aws_secret_access_key\s*=\s*(.*)$/) {
+            last if defined $creds->{aws_secret_access_key};    # next profile
+            $creds->{aws_secret_access_key} = $1;
+          }
+          elsif ($current_line =~/^\s*aws_access_key_id\s*=\s*(.*)$/) {
+            last if defined $creds->{aws_access_key_id};        # next profile
+            $creds->{aws_access_key_id} = $1;
+          }
+          elsif ($current_line =~/^\s*aws_session_token\s*=\s*(.*)$/) {
+            last if defined $creds->{token};
+            $creds->{token} = $1;
+          }
+        } ## end while (<$fh>)
+
+        close $fh;
+
+        $self->get_logger->debug( Dumper [$creds] );
+
+        $creds->{source} = $config
+          if $creds->{aws_secret_access_key} && $creds->{aws_access_key_id};
+      } ## end foreach my $config ( ".aws/config"...)
+    } ## end elsif ( $location eq 'file')
+
+    last if $creds->{source};
+  } ## end foreach my $location (@order)
+
+  foreach my $k ( keys %$creds ) {
+    $self->set( $k, $creds->{$k} );
   }
-  
-  foreach ( keys %$creds) {
-    $self->set($_, $creds->{$_});
-  }
-  
+
   return $creds;
-}
+} ## end sub find_credentials
 
 =pod
 
@@ -460,8 +482,7 @@ sub is_token_expired {
     
     $expired = ($seconds_left < 0) ? 1 : 0;
     
-    $self->get_logger->debug(Dumper [ "EXPIRATION TIME: " . $expiration_date, "EXPIRED: " . $expired])
-      if $self->get_debug;
+    $self->get_logger->debug(Dumper [ "EXPIRATION TIME: " . $expiration_date, "EXPIRED: " . $expired]);
   }
 
   return $expired;
@@ -539,27 +560,24 @@ sub get_creds_from_role {
       
       my $req = HTTP::Request->new( GET => $url );
       
-      $self->get_logger->debug(Dumper [ "HTTP REQUEST:\n", $req ])
-	if $self->get_debug;
+      $self->get_logger->debug(Dumper [ "HTTP REQUEST:\n", $req ]);
       
       my $rsp = $ua->request($req);
       
-      $self->get_logger->debug(Dumper [ "HTTP RESPONSE:\n", $rsp ])
-	if $self->get_debug;
+      $self->get_logger->debug(Dumper [ "HTTP RESPONSE:\n", $rsp ]);
       
       # if not 200, then get out of Dodge
       last unless $rsp->is_success;
       
       if ( $role ) {
 	$creds->{serialized} = $rsp->content;
-	my $this = from_json($creds->{serialized});
+	my $this = JSON::PP->new->utf8->decode($creds->{serialized});
 	@{$creds}{qw/source role aws_access_key_id aws_secret_access_key token expiration/} =
 	  ('IAM',$role, @{$this}{qw/AccessKeyId SecretAccessKey Token Expiration/});
       }
       else {
 	$role = $rsp->content;
-	$self->get_logger->debug(Dumper ['role', $role])
-	  if $self->get_debug;
+	$self->get_logger->debug(Dumper ['role', $role]);
 
 	last unless $role;
       }
@@ -601,8 +619,7 @@ sub refresh_token {
     $creds = $self->get_creds_from_role;
   }
 
-  $self->get_logger->debug(Dumper [$creds])
-    if $self->get_debug;
+  $self->get_logger->debug(Dumper [$creds]);
 
   croak 'unable to refresh token!'
     if !ref($creds) || !keys %$creds;
@@ -638,6 +655,8 @@ sub get_creds_from_container {
   my $creds = {};
 
   if ($uri) {
+    $self->get_logger->debug(caller(2), $uri);
+    
     eval {
       # try to get credentials from instance role
       my $url = AWS_CONTAINER_CREDENTIALS_URL . $uri;
@@ -646,23 +665,20 @@ sub get_creds_from_container {
       my $req = HTTP::Request->new( GET => $url );
       $req->header( "Accept", "*/*" );
 
-      $self->get_logger->debug( Dumper [ "HTTP REQUEST:\n", $req ] )
-        if $self->get_debug;
+      $self->get_logger->debug( Dumper [ "HTTP REQUEST:\n", $req ] );
 
-      $self->get_logger->debug( Dumper [ $req->as_string ] )
-        if $self->get_debug;
+      $self->get_logger->debug( Dumper [ $req->as_string ] );
 
       my $rsp = $ua->request($req);
 
-      $self->get_logger->debug( Dumper [ "HTTP RESPONSE:\n", $rsp ] )
-        if $self->get_debug;
+      $self->get_logger->debug( Dumper [ "HTTP RESPONSE:\n", $rsp ] );
 
       # if not 200, then get out of Dodge
       if ( $rsp->is_success ) {
 
         $creds->{serialized} = $rsp->content;
 
-        my $this = from_json( $rsp->content );
+        my $this = JSON::PP->new->utf8->decode( $rsp->content );
 
         @{$creds}{
           qw/source container aws_access_key_id aws_secret_access_key token expiration/
@@ -692,7 +708,7 @@ Rob Lauer - <rlauer6@comcast.net>
 
 =head1 LICENSE
 
-(c) Copyright 2019 Robert C. Lauer. All rights reserved.  This module
+(c) Copyright 2022 Robert C. Lauer. All rights reserved.  This module
 is free software. It may be used, redistributed and/or modified under
 the same terms as Perl itself.
 
